@@ -10,7 +10,7 @@ import math
 matplotlib.use('Agg')
 
 
-def arrNumeric(arr, typ):
+def arrNumeric(arr, typ, palette='CBS'):
     '''
     Converts the sequence array into a numerical matrix and a colour map
     which matplotlib can interpret as an image (similar to
@@ -26,6 +26,9 @@ def arrNumeric(arr, typ):
     typ: str
         Either 'aa' - amino acid - or 'nt' - nucleotide
 
+    palette: str
+        Colour palette, CBS or Bright
+
     Returns
     -------
     arr2: np.array
@@ -37,9 +40,9 @@ def arrNumeric(arr, typ):
     # turn the array upside down
     arr = np.flip(arr, axis=0)
     if typ == 'nt':
-        D = utilityFunctions.getNtColours()
+        D = utilityFunctions.getNtColours(palette)
     else:
-        D = utilityFunctions.getAAColours()
+        D = utilityFunctions.getAAColours(palette)
 
     # retrieve the colours for the colour map
     keys = list(D.keys())
@@ -66,7 +69,8 @@ def arrNumeric(arr, typ):
     return (arr2, cmap)
 
 
-def drawMarkUp(a, markupdict, nams, ali_width, ali_height):
+def drawMarkUp(a, markupdict, nams, ali_width, ali_height,
+               palette='CBS'):
     '''
     Add the "markup" to the mini alignment - on the input alignment image
     use coloured lines to show which rows, columns and positions have
@@ -93,13 +97,27 @@ def drawMarkUp(a, markupdict, nams, ali_width, ali_height):
         The number of columns in the input alignment
     ali_height: int
         The number of rows (sequences) in the input alignment
+    palette: str
+        Colour palette, CBS or Bright
 
     Returns
     -------
 
     '''
-    colD = utilityFunctions.getMarkupColours()
+    colD = utilityFunctions.getMarkupColours(palette)
     lineweight_h = 5 / ali_height
+
+    # removes columns
+    if "user" in markupdict:
+        colour = colD['user']
+        for col in markupdict['user']:
+
+            a.add_patch(matplotlib.patches.Rectangle(
+                    (col-0.5, -0.5), 1, ali_height, color=colour, zorder=60,
+                    lw=0))
+            for row in np.arange(ali_height):
+                a.hlines(row, col-0.5, col+0.5, zorder=61, color='black',
+                         lw=lineweight_h)
     # removes single positions
     if "crop_ends" in markupdict:
         colour = colD['crop_ends']
@@ -126,7 +144,6 @@ def drawMarkUp(a, markupdict, nams, ali_width, ali_height):
                 a.hlines(y, boundary[1][0]-0.5, boundary[1][-1]+0.5,
                          zorder=51, color='black',
                          lw=lineweight_h)
-
     # removes whole rows
     if "remove_divergent" in markupdict:
         colour = colD['remove_divergent']
@@ -147,9 +164,21 @@ def drawMarkUp(a, markupdict, nams, ali_width, ali_height):
             a.add_patch(matplotlib.patches.Rectangle(
                     (col-0.5, -0.5), 1, ali_height, color=colour, zorder=46,
                     lw=0))
-            for row in np.arange(ali_height):
-                a.hlines(row, col-0.5, col+0.5, zorder=47, color='black',
-                         lw=lineweight_h)
+            a.hlines(np.arange(ali_height), col-0.5, col+0.5, zorder=47,
+                     color='black',
+                     lw=lineweight_h)
+
+    # removes whole columns
+    if "crop_divergent" in markupdict:
+        colour = colD['crop_divergent']
+        for col in markupdict['crop_divergent']:
+
+            a.add_patch(matplotlib.patches.Rectangle(
+                    (col-0.5, -0.5), 1, ali_height, color=colour, zorder=46,
+                    lw=0))
+            a.hlines(np.arange(ali_height), col-0.5, col+0.5, zorder=47,
+                     color='black',
+                     lw=lineweight_h)
 
     # removes whole rows
     if "remove_short" in markupdict:
@@ -162,9 +191,9 @@ def drawMarkUp(a, markupdict, nams, ali_width, ali_height):
             a.hlines(y+0.5, -0.5, ali_width-0.5, zorder=45, color='black',
                      lw=lineweight_h)
     # removes whole columns
-    if "remove_gaponly" in markupdict:
-        colour = colD['remove_gaponly']
-        for col in markupdict['remove_gaponly']:
+    if "remove_gap_only" in markupdict:
+        colour = colD['remove_gap_only']
+        for col in markupdict['remove_gap_only']:
             a.add_patch(matplotlib.patches.Rectangle((col-0.5, -0.5), 1,
                                                      ali_height,
                                                      color=colour,
@@ -174,7 +203,7 @@ def drawMarkUp(a, markupdict, nams, ali_width, ali_height):
                          lw=lineweight_h)
 
 
-def drawMarkUpLegend(outfile):
+def drawMarkUpLegend(outfile, palette="CBS"):
     '''
     Draws a small legend (in a seperate image) showing which parsing step
     each colour represents in the marked up mini alignment.
@@ -186,25 +215,29 @@ def drawMarkUpLegend(outfile):
     ----------
     outfile: str
         Path to the output file
-
+    palette: str
+        Colour palette, CBS or Bright
     Returns
     -------
     None
     '''
     legend = plt.figure(figsize=(2, 2), dpi=100)
     leg = legend.add_subplot(1, 1, 1)
-    colours = utilityFunctions.getMarkupColours()
+    colours = utilityFunctions.getMarkupColours(palette)
 
     functions = {'crop_ends': 'Cropped Ends',
                  'remove_divergent': 'Too Divergent',
                  'remove_insertions': 'Insertions',
                  'remove_short': 'Too Short',
-                 'remove_gaponly': 'Gap Only'}
+                 'remove_gap_only': 'Gap Only',
+                 'crop_divergent': 'Crop Divergent',
+                 'user': 'User'}
+    L = len(functions)
     for i, (func, txt) in enumerate(functions.items()):
-        leg.plot(1, 5-i, marker='.', color=colours[func], markersize=20)
-        leg.text(2, 5-i, txt)
+        leg.plot(1, L-i, marker='.', color=colours[func], markersize=20)
+        leg.text(1.5, L-i, txt, va='center', ha='left')
     leg.set_xlim(0.5, 3)
-    leg.set_ylim(-1, 6)
+    leg.set_ylim(L-i-1, L+1)
     leg.set_axis_off()
     legend.gca().set_axis_off()
     leg.margins(0, 0)
@@ -215,7 +248,7 @@ def drawMarkUpLegend(outfile):
 def drawMiniAlignment(arr, nams, log, outfile, typ,
                       dpi=300, title=None, width=5, height=3, markup=False,
                       markupdict=None, ret=False, orig_nams=[],
-                      keep_numbers=False, force_numbers=False):
+                      keep_numbers=False, force_numbers=False, palette="CBS"):
     '''
     Draws a "mini alignment" image showing a small representation of the
     whole alignment so that gaps and poorly aligned regions are visible.
@@ -254,7 +287,8 @@ def drawMiniAlignment(arr, nams, log, outfile, typ,
     keep_numbers: bool
         Number the sequences (rows) based on the original CIAlign input rather
         than renumbering.
-
+    palette: str
+        Colour palette, CBS or Bright
     Returns
     -------
     None
@@ -283,7 +317,7 @@ def drawMiniAlignment(arr, nams, log, outfile, typ,
     a.set_ylim(-0.5, ali_height-0.5)
 
     # generate the numeric version of the array
-    arr2, cm = arrNumeric(arr, typ)
+    arr2, cm = arrNumeric(arr, typ, palette)
     # display it on the axis
     a.imshow(arr2, cmap=cm, aspect='auto', interpolation='nearest')
 
@@ -302,7 +336,7 @@ def drawMiniAlignment(arr, nams, log, outfile, typ,
     a.spines['left'].set_visible(False)
 
     if title:
-        f.suptitle(title)
+        f.suptitle(title, fontsize=fontsize*1.5, y=0.92)
     for t in a.get_xticklabels():
         t.set_fontsize(fontsize)
     a.set_yticks(np.arange(ali_height-1, -1, -tickint))
@@ -315,16 +349,16 @@ def drawMiniAlignment(arr, nams, log, outfile, typ,
                     labs.append(x)
                 x += 1
             a.set_yticklabels(labs,
-                              fontsize=fontsize)
+                              fontsize=fontsize*0.75)
         else:
             a.set_yticklabels(np.arange(1, ali_height+1, tickint),
-                              fontsize=fontsize)
+                              fontsize=fontsize*0.75)
     else:
         a.set_yticklabels(np.arange(0, ali_height, tickint), fontsize=fontsize)
 
     if markup:
-        a = drawMarkUp(a, markupdict, nams, ali_width, ali_height)
-        drawMarkUpLegend(outfile.replace(".png", ""))
+        a = drawMarkUp(a, markupdict, nams, ali_width, ali_height, palette)
+        drawMarkUpLegend(outfile.replace(".png", ""), palette)
     f.tight_layout()
     f.savefig(outfile, dpi=dpi, bbox_inches='tight')
     if ret:
