@@ -5,6 +5,7 @@ import matplotlib.pyplot as plt
 import warnings
 import matplotlib.font_manager
 import sys
+import os
 try:
     import CIAlign.palettes as palettes
 except ImportError:
@@ -12,7 +13,7 @@ except ImportError:
 matplotlib.use('Agg')
 
 
-def replaceUbyT(arr):
+def replaceUbyT(arr, rev):
     '''
     Replaces all Us by Ts in the alignment.
 
@@ -26,8 +27,10 @@ def replaceUbyT(arr):
     arr: np.array
         2D numpy array of sequences with Ts instead of Us.
     '''
-
-    arr = np.where(arr == "U", "T", arr)
+    if not rev:
+        arr = np.where(arr == "T", "U", arr)
+    else:
+        arr = np.where(arr == "U", "T", arr)
     return (arr)
 
 
@@ -80,7 +83,7 @@ def FastaToArray(infile, log=None, outfile_stem=None):
         for line in input:
             line = line.strip()
             if len(line) == 0:
-                continue  # todo: test!
+                continue
             if line[0] == ">":
                 seqs.append([s.upper() for s in seq])
                 nams.append(nam)
@@ -91,7 +94,7 @@ def FastaToArray(infile, log=None, outfile_stem=None):
                     if log:
                         log.error(formatErrorMessage)
                     print(formatErrorMessage)
-                    exit()
+                    exit(1)
                 seq += list(line)
     seqs.append(np.array([s.upper() for s in seq]))
     nams.append(nam)
@@ -118,11 +121,12 @@ def getPalette(palette='CBS'):
     '''
     if palette.lower() == 'cbs':
         p = palettes.CBSafe()
-
+    if palette.lower() == 'bright':
+        p = palettes.Bright()
     return (p)
 
 
-def getAAColours(pal='CBS'):
+def getAAColours(palette='CBS'):
     '''
     Generates a dictionary which assigns a colour to each amino acid.
     Based on the "RasmMol" amino colour scheme and the table here:
@@ -142,7 +146,7 @@ def getAAColours(pal='CBS'):
         Dictionary where keys are single letter amino acid codes and
         values are hexadecimal codes for colours
     '''
-    pal = getPalette(palette=pal)
+    pal = getPalette(palette=palette)
     return {'D': pal['red_aa'],
             'E': pal['red_aa'],
             'C': pal['yellow_aa'],
@@ -174,7 +178,7 @@ def getAAColours(pal='CBS'):
             }
 
 
-def getNtColours(pal='CBS'):
+def getNtColours(palette='CBS'):
     '''
     Generates a dictionary which assigns a colour to each nucleotide (plus grey
     for "N" and white for "-")
@@ -190,7 +194,7 @@ def getNtColours(pal='CBS'):
         Dictionary where keys are single letter nucleotide codes and
         values are hexadecimal codes for colours
     '''
-    pal = getPalette(palette=pal)
+    pal = getPalette(palette=palette)
     return {'A': pal['green_nt'],
             'G': pal['yellow_nt'],
             'T': pal['red_nt'],
@@ -229,9 +233,11 @@ def getMarkupColours(pal='CBS'):
     pal = getPalette(palette=pal)
     return {'remove_insertions': pal['remove_insertions'],
             'crop_ends': pal['crop_ends'],
-            'remove_gaponly': pal['remove_gaponly'],
+            'remove_gap_only': pal['remove_gap_only'],
             'remove_short': pal['remove_short'],
-            'remove_divergent': pal['remove_divergent']}
+            'remove_divergent': pal['remove_divergent'],
+            'crop_divergent': pal['crop_divergent'],
+            'user': pal['user']}
 
 
 def writeOutfile(outfile, arr, nams, removed, rmfile=None):
@@ -265,7 +271,7 @@ def writeOutfile(outfile, arr, nams, removed, rmfile=None):
     out.close()
 
 
-def seqType(arr):
+def seqType(arr, log):
     '''
     Detects if an alignment is of nucleotides or amino acids using pre-built
     dictionarys of amino acid and nucleotide codes.
@@ -306,16 +312,15 @@ def seqType(arr):
             aa_count += 1
             ch += 1
         if ch == 0:
+            log.error("Unknown nucleotides or amino acids detected.\
+                       Please fix your MSA.")
             print("Unknown nucleotides or amino acids detected.\
                   Please fix your MSA.")
-            exit()
-
+            exit(1)
     if nt_count == len(arr):
         return "nt"
     if aa_count == len(arr):
         return "aa"
-    print("MSA type couldn't be established. Please fix your MSA.")
-    exit()
 
 
 def updateNams(nams, removed_seqs):
@@ -366,11 +371,11 @@ not all the same length."""
     if 0 in np.shape(arr):
         log.error(emptyAlignmentMessage)
         print(emptyAlignmentMessage)
-        exit()
+        exit(1)
     if len(np.shape(arr)) == 1:
         log.error(differentLengthsMessage)
         print(differentLengthsMessage)
-        exit()
+        exit(1)
 
 
 def listFonts(outfile):
@@ -439,3 +444,201 @@ def listFonts(outfile):
 
         f.tight_layout()
         f.savefig(outfile, dpi=200, bbox_inches='tight')
+
+
+def configRetainSeqs(retain, retainS, retainL, nams, fname, log, silent):
+    '''
+    Allows the user to specify sequences to keep regardless of whether
+    they pass or fail the rowwise cleaning operation thresholds. This function
+    works on the sequences specified for a single function or group of
+    functions - either all rowwise functions, crop ends, remove divergent
+    or remove short.
+
+    Sequence names can be specified individually on the command line with
+    --retain, --crop_ends_retain, --remove_divergent_retain,
+    --remove_short_retain - these are listed in the retain variable for
+    the specific function currently being processed.
+
+    They can also be specified as a list in a text file with --retain_list,
+    --crop_ends_retain_list etc, in which case the value is the path to
+    the file.
+
+    Finally they can be specified by searching each name for a character
+    string, specified as --retain_str, --crop_ends_retain_str etc.
+
+    retain, retainS and retainL are None if no sequences are specified.
+
+    Parameters
+    ----------
+    retain: list
+        List of sequence names to keep
+    retainS: str
+        Sequence names containing this string will be kept
+    retainL: str
+        Path to a text file containing a list of sequence names to keep
+    nams: np.array
+        Array containing the names of the sequences in the input fasta file
+    fname: str
+        The function currently being processed - all_rowwise, crop_ends,
+        remove_divergent or remove_short, used for logging only here.
+    log: logging.Logger
+        An open log file object
+    silent: bool
+        True if CIAlign is run in silent mode - nothing will be printed to
+        STDOUT
+
+    Returns
+    -------
+    keeps: np.array
+        An array containing all sequence names to be ignored by this
+        function
+    '''
+    # First read the sequence names passed directly
+    # the len check is for users with an inifile - can't specify None
+    # so this will be an empty string
+    if retain is not None and len(retain[0].strip()) != 0:
+        keeps = set(retain)
+    else:
+        keeps = set()
+
+    # If a file is specified, read the sequence names in the file
+    if retainL is not None and len(retainL.strip()) != 0:
+        # Raise an error if the file is not found
+        if not os.path.exists(retainL):
+            raise FileNotFoundError("""
+List of sequences to retain %s not found""" % retainL)
+
+        with open(retainL) as infile:
+            for line in infile:
+                keeps.add(line.strip())
+
+    # If a string to match is specified
+    if retainS is not None and len(retainS[0].strip()) != 0:
+        # For each string
+        for rs in retainS:
+            rr = 0
+            # check all the sequence names for this string
+            for nam in nams:
+                if rs in nam:
+                    keeps.add(nam)
+                    rr += 1
+            if rr == 0:
+                # Warn if there are no matches
+                log.warning("""No sequence names matching "%s" were found"""
+                            % rs)
+                if not silent:
+                    print("""Warning: No sequence names matching \
+"%s" were found""" % rs)
+
+    # Check all the specified sequence names were found, raise an error if
+    # not
+    if len(keeps & set(nams)) != len(keeps):
+        raise RuntimeError("""
+Some sequences listed to be retained were not found: %s""" % (" ".join(
+                                                              keeps - set(
+                                                                  nams))))
+
+    # Convert the result to an array
+    keeps_arr = np.array(sorted(list(keeps)))
+
+    # Log the sequence names identified
+    if len(keeps_arr) != 0:
+        log.info("""The following sequences will not be processed with the \
+%s function: %s""" % (fname, ", ".join(sorted(list(keeps)))))
+
+    return (keeps_arr)
+
+
+def updateStartEnd(start, end, removed):
+    '''
+    Updates the start and end positions of a subsection of an array taking
+    into account positions that have been removed i.e. if the user
+    wants columns 10:20 of the input FASTA but 5 and 15 have been removed,
+    then in the updated alignment they would want 9:18.
+
+    Parameters
+    ----------
+    start: int
+        The start position in the unedited alignment
+    end: int
+        The end position in the unedited alignment
+    removed: set
+        Set of integers representing positions which have been removed from
+        the alignment
+
+    Returns
+    -------
+    newstart: int
+        The updated start position
+    newend: int
+        The updated end position
+    '''
+    newstart = start
+    newend = end
+    if len(removed) != 0:
+        for r in sorted(removed):
+            if r < newstart:
+                newstart -= 1
+                newend -= 1
+            elif r < newend:
+                newend -= 1
+    return (newstart, newend)
+
+
+def removeColumns(rmAbsolute, relativePositions,
+                  arr, log, rmfile, function_name, write=True):
+    '''
+    Function to remove a column from an array shared by removeInsertions,
+    removeGapOnly. Removes the columns, calculates the
+    positions of these columns in the input alignment, writes positions
+    removed to the log file and the removed file.
+
+    Parameters
+    ----------
+    rmAbsolute: list
+        List of absolute positions (positions in current alignment) to
+        be removed
+    relativePositions: list
+        A list of integers representing columns in the alignment, from which
+        values are removed as columns are removed from the alignment.
+    arr: np.array
+        The alignment stored as a numpy array
+    log: logging.Logger
+        An open log file object
+    rmfile: str
+        Path to a file in which to store a list of removed sequences
+    function_name: str
+        The name of the function which removed these columns for logging
+    Returns
+    -------
+    arr: np.array
+        The cleaned alignment stored as a numpy array
+    relativePositions: list
+        A list of integers representing columns in the alignment, from which
+        values are removed as columns are removed from the alignment, minus
+        the columns removed using this function.
+    r: set
+        A set of column numbers of sequences which have been removed
+    '''
+    outrm = open(rmfile, "a")
+    if outrm.closed:
+        print('file is closed')
+    # make a list of positions to remove
+    rm_relative = set()
+    for n in rmAbsolute:
+        rm_relative.add(relativePositions[n])
+    for n in rm_relative:
+        relativePositions.remove(n)
+    # for n in absolutePositions:
+    #     relativePositions.remove(n)
+    rmpos = np.array(list(rmAbsolute))
+
+    keeppos = np.arange(0, np.shape(arr)[1])
+    keeppos = np.invert(np.in1d(keeppos, rmpos))
+    if len(rmpos) != 0 and write:
+        rmpos_str = [str(x) for x in sorted(rm_relative)]
+        log.info("Removing sites %s" % (", ".join(rmpos_str)))
+        outrm.write("%s\t%s\n" % (function_name, ",".join(rmpos_str)))
+    outrm.close()
+    arr = arr[:, keeppos]
+    return (arr, relativePositions, rm_relative)
